// autor: Ivan Mosquera Paulo 
// email: ivanmosp@gmail.com
// nombreInterprete: euiti-bi_LSI_FI_C
// version:	1.0
// extensiones:	c cpp
//

// ==================== FuncionEval.g : fichero de intrprete AST, primera pasada =======================

// En este fichero lo que hacemos es hacer una primera interpretacin del rbol AST generado desde 
// el analizador sintctico.

// En este fichero lo que hacemos es ver qu hay en cada momento en la rama del rbol AST en que estamos
// y generamos el cdigo Java del intrprete correspondiente. 
// La parte izquierda de las reglas es idntica a la parte derecha de las reglas de los analizadores.
// Esto es as porque en los analizadores en la parte derecha definamos qu AST generabamos y ahora
// en la parte izquiera definimos qu parte del AST estamos mirando. 

tree grammar FuncionEval;

options {
	tokenVocab = C; // Se utilizarn los tokens definidos en la gramtica C.g
	ASTLabelType = CommonTree;
}

// Aadimos lo que necesitemos.
@header {
	package ikuspro.interprete.c;
	import ikuspro.interprete.c.*;
	import ikuspro.interprete.c.ParametroFuncion;
	import java.util.List;
	import java.util.ArrayList;
}

// Solo se evaluan las funciones para poder declararlas.
programa 
	:	^(PROGRAMA ^(DECLARACIONES_GLOBALES .*) ^(FUNCIONES funciones) .) 
	|	^(PROGRAMA ^(FUNCIONES funciones) .)
	; 

funciones 
	:	
	funcion+
	;
	

// Una regla puede devolver estructuras de datos que pueden ser utilizadas como informacin por otras reglas.
// En este caso devolvemos un String con el tipo.
tipo_base returns [String valor]
	:	'void' { $valor = "void"; }
	|	'char' { $valor = "char"; }
	| 	'int' { $valor = "int"; }
	|	'float' { $valor = "float"; }
	|	'double' { $valor = "float"; } // Por simplicidad en este punto hacemos que un double sea en la prctica un float. 
	|	'string' { $valor = "string"; }
	;

// Se declaran las funciones.
funcion
scope {
	List<ParametroFuncion> listaParametros; // Necesitamos una lista donde guardar los parmetros de la funcin.
	String nombre;
	String tipo;
	int ini;
	int fin;
}

// En @init definimos el cdigo que se inserta antes de empezar la regla.
@init {
	$funcion::listaParametros = new ArrayList<ParametroFuncion>();
}

// En @after definimos el cdigo que se inserta tras haber terminado la regla.
// Como en este punto es cuando tenemos toda la informacin, es donde hacemos lo ms interesante.
@after {
	// Guardamos en una variable el nodo que nos interesa del rbol.
	CommonTree nodoBloque = (CommonTree) $funcion.start.getChild(1);
	//System.out.println("INDEX : " + String.valueOf(input.index())); 
	int i = $funcion::ini;
	boolean encontrado = false;
	
	// Sabemos el nodo que queremos pero no sabemos el ndice.
	// Lo que s sabemos son los ndices entre los que acotar la bsquera, inicio y fn
	// de la funcin, y tambin sabemos lo que hay que buscar. 
	// Menos mal porque de lo contrario sera inviable por lo ineficiente.
	// En versiones anteriores de ANTLR haba mtodos que hacan esto directamente pero ya no por
	// cuestiones de eficiencia. De hecho muchos ejemplos de intrpretes que hay en la WEB para 
	// Antlr no son funcionales por esta razn.
	// Es posible que haya una forma ms eficiente de hacer esto pero yo no la he encontrado.
	while(!encontrado && i <= $funcion::fin) {
		if (input.get(i) == nodoBloque) {
			encontrado = true;
		} else {
			i++;
		}
	}
	// Los System.out residuales no son recomendables pero en algunos casos los mantenemos comentados
	// porque son tremendamente tiles de cara a debuggear el intrprete.
	//System.out.println("TREE : " + nodoBloque.toStringTree());
	
	// Aqu es donde finalmente declaramos la funcin con la informacin que tenemos.
	Interprete.getTablaSimbolos().declararFuncion($funcion::nombre,$funcion::tipo, $funcion::listaParametros, nodoBloque, i);
	
}
	: ^(FUNCION { $funcion::ini = input.index();} ^(CABECERA_FUNCION cabecera_funcion  ) { $funcion::fin = input.index();}  .)  
	
	// Un par de detalles:
	// Tambin podemos insertar cdigo dentro de la regla, en esta regla por ejemplo necesitaremos el indice donde empieza
	// la funcin y obviamente eso lo sabemos en el momento que estamos en ese ndice.
	// Del mismo modo necesitaremos saber el indice de donde termina la funcin.
	// El "." que vemos al final lo que hace es ignorar parte del rbol, en este caso ignoramos el bloque de la funcin
	// porque en esta primera pasada del rbol AST no vamos a mirar ah.
	
	;
	
cabecera_funcion returns [String nombre, String tipo] 
	:	
	declaracion_nombre_funcion lista_param_formales? 
	{ $funcion::nombre = $declaracion_nombre_funcion.nombre; $funcion::tipo = $declaracion_nombre_funcion.tipo; }
		
	;

declaracion_nombre_funcion returns [String nombre, String tipo]
	:	^(DECLARACION_NOMBRE_FUNCION ^(NOMBRE_FUNC IDENT)) { $nombre = $IDENT.text; $tipo = "void"; $funcion::tipo = $tipo;}
	|	^(DECLARACION_NOMBRE_FUNCION ^(TIPO_BASE tipo_base ) ^(NOMBRE_FUNC IDENT)) 
					{ $nombre = $IDENT.text; $tipo = $tipo_base.valor; $funcion::tipo = $tipo;}
	;
	
lista_param_formales 
	:	param_formal ( param_formal )*
	;

// En esta regla se aaden los parmetros. Vemos que se utiliza la variable global de la regla funcion.
// Hay que entender el orden en que se van ejecutando estas reglas. 
// Por ejemplo cuando estamos en funcion, estar formado por parmetros formales, y cuando estamos en esta 
// regla de parmetros formales en realidad todava estamos en el mbito de la regla funcion, la regla funcion
// no ha terminado por lo que podemos darle informacin que luego utilizar en su @after por ejemplo.	
param_formal
	:	
		^(PARAM_FORMAL ^(TIPO_BASE tipo_base) ^(NOMBRE_VAR IDENT) )
		{ $funcion::listaParametros.add(new ParametroFuncion($IDENT.text,$tipo_base.valor)); 
		}
	|	^(PARAM_FORMAL_ARR ^(TIPO_BASE tipo_base) ^(NOMBRE_VAR IDENT) )
		{ $funcion::listaParametros.add(new ParametroFuncion($IDENT.text,$tipo_base.valor,true)); 
		  //System.out.println("El parametro es un array");
		}
	
	;
